#ifndef __CLF_WORLD__
#define __CLF_WORLD__

#include "clf_math.h"

// base object type
class clf_world_object
{
public:
	static clf_vector3D gravity; // the famous g

	float m_mass;				// object mass
	float m_radius;     // bounding sphere
	float m_radius2;		// radius squared to speed up distance calculations
	clf_vector3D m_pos; // position
	clf_vector3D m_vel; // velocity
	bool m_fixed_pos;

	clf_world_object(const float m, const float r );
	virtual ~clf_world_object();
	virtual void step_simulation(float fElapsed);
	virtual void render();
	// check for collision pure virtual function
	virtual bool collides_with(clf_world_object* pob)=0;
	virtual bool is_at_xz(clf_vector3D& pos, float r);
};

// our game world
class clf_world
{
	static float sim_speed; // used to control game speed, 1 is realtime <1 slow motion, >1 fast
public:
	// a list of the world objects
	std::list<clf_world_object*> m_objects;
	clf_world();
	~clf_world();
	// iterate the objects and find collisions
	void collision_detection();
	// calculate next frame
	void step_simulation(float fElapsed);
	// draw the objects
	void render();
	// add object to world
	void add_object(clf_world_object* pob);

	clf_world_object* object_at(clf_vector3D& pos, float r);
};

// the sphere object
class clf_sphere : public clf_world_object
{
public:
	clf_sphere(float m, float r );
	virtual ~clf_sphere();
	virtual void render();
	virtual void step_simulation(float fElapsed);
	void collision_respond(clf_sphere* pob);
	virtual bool collides_with(clf_sphere* pob);
	virtual bool collides_with(clf_world_object* pob);
};

#endif // __CLF_WORLD__